#!/ffp/bin/sh
# ============================================================================
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Author: Mijzelf <Mijzelf@live.com>
#
# ============================================================================
#
# This script has multiple functions.
# - Change homedir for any user
# - Change shell for any user
# - Enable shell access for any user
# - Inject clean shutdown code in /etc/init.d/rc.shutdown
# - Change http port of firmware webserver
# - Change https port of firmware webserver
# - Hook nfs. Use /ffp/etc/exports instead of /etc/exports
# - Hook ftp to force it to advertise an external ip address
# - Hook readonly directories to make them read-write
# - Hook crond to include /ffp/etc/crontab
# - Hook any binary to log it's calls.
#
# More info http://zyxel.nas-central.org/w/index.php?title=Zyxel_utils

# PROVIDE: zyxel_utils
# REQUIRE: LOGIN
# BEFORE: lighttpd

. /ffp/etc/ffp.subr
name=zyxel_utils

SED=sed
INTERCEPT_LOG=/tmp/intercept.log

zyxel_utils()
{
	##########################################################
	## Change homedir of any user.
	##########################################################
	change_homedir root /ffp/home/root
	change_homedir admin /ffp/home/admin
	
	##########################################################
	## Change shell of any user
	##########################################################
	# change_shell nobody /ffp/bin/sh
	
	##########################################################
	## Enable shell access for any user. If you enable this,
	## you can change the password for any user by executing
	## change_password <any-user>
	## The password will be restored on next boot
	##
	## Code courtesy of arnova
	##########################################################
	# enable_shell_access 
	
	##########################################################
	## Inject a ffp stop command in /etc/init.d/rc.shutdown
	## to give daemons the possibility to close cleanly
	##########################################################
	hook_shutdown
	
	##########################################################
	## Change the http port of the firmware webserver, to free
	## it up for another webserver
	##########################################################
	# change_http_port 81
	
	##########################################################
	## Change the https port of the firmware webserver, to 
	## free it up for another webserver
	## Note: Do not use this in combination with the 
	## 'force https' feature in the administration menu
	##########################################################
	# change_https_port 444
	
	##########################################################
	## Use /ffp/etc/samba/smb.conf instead of 
	## /etc/samba/smb.conf
	## 
	## This gives the possibility to maintain your own smb.conf
	##
	## Note: When this option is active, you can't change any
	## setting using the webinterface
	##########################################################
	# hook_samba
	
	##########################################################
	## Force the ftp daemon to use the public extern ip address
	## instead of the local one.
	## (This will also hook_directory /usr/local/sbin/)
	##########################################################
	# hook_ftp myname.dyndns.com
	
	##########################################################
	## Use /ffp/etc/exports instead of /etc/exports
	##########################################################
	# hook_nfs
	
	##########################################################
	## The directories under /usr are mostly readonly, while
	## you might want to change some files, for instance 
	## twonky settings
	## Make them writeable:
	##########################################################
	# hook_directory /usr/local/dmsf
	
	##########################################################
	## Let the firmware crond handle your cronjobs, which are 
	## stored in /ffp/etc/crontab
	##########################################################
	# hook_crond
}

STOREDIR=/tmp/.ffpstick/
DBFILE=${STOREDIR}zyxelutils

read_db()
{
	local header=$1
	local value=""
	
	[ -f $DBFILE ] && value=` grep "^$header" $DBFILE | awk '{ print $3 }' `

#	echo "read_db $header => $value" >&2
	echo $value
}

write_db()
{
	local header=$1
	local value=$2
#	echo write_db $header $value >&2

	[ -f $DBFILE ] && grep -v "^$header" $DBFILE >$DBFILE.new && mv $DBFILE.new $DBFILE
	echo "$header $value" >>$DBFILE
	chmod 600 $DBFILE
}

KICKWEBS=0

change_httpdconf() 
{
	local newvalue=$1
	local oldvalue=$2
	local valuename=$3
	local prefix="\tchange_${valuename}_port $1:"
	
	local httpd_conf=/etc/original/service_conf/httpd_zld.conf
	[ "${CHROOTDIR}" = "" ] && httpd_conf=/etc/service_conf/httpd_zld.conf
	

	if [ "$ACTION" = "stop" ] ; then
		local store=$oldvalue
		oldvalue=$newvalue
		newvalue=$store
	fi

	[ "$oldvalue" = "$newvalue" ] && echo -e "$prefix port is already $newvalue. Won't change" && return 0

	local fmt="s|Listen $oldvalue\$|Listen $newvalue|g"
	${SED} -i "$fmt" $httpd_conf
	[ $? -ne 0 ] && return 0
	fmt="s|<VirtualHost \*:$oldvalue>\$|<VirtualHost \*:$newvalue>|g"
	${SED} -i "$fmt" $httpd_conf
	[ $? -eq 0 ] && echo -e "$prefix changed $valuename port to $newvalue" && KICKWEBS=1
}

change_https_port()
{
    change_httpdconf $1 443 https
}

change_http_port()
{
    change_httpdconf $1 80 http
}

enable_shell_access()
{
    local prefix="\tenable_shell_access:"
    local symlink="/ffp/sbin/change_password"
    
    if [ "$ACTION" = "stop" ] ; then
	if [ -h $symlink ] ; then 
	    rm $symlink
	    echo -e "$prefix Disabled"
	fi
	return
    fi
    
    [ "$ACTION" != "start" ] && return


    ln -s ` this_file ` $symlink
    echo -e "$prefix Enabled"

    unset IFS
    grep -E -v "^[a-z\-]+:[\!\*]" /ffp/etc/shadow |grep -v -e '^root:' -e '^password' |while read LINE; do

	USER_NAME="$(echo "$LINE" |cut -d: -f1)"
	PASSWD_HASH="$(echo "$LINE" |cut -d: -f2)"

	sed -i s,"^${USER_NAME}:!:","${USER_NAME}:${PASSWD_HASH}:", /etc/shadow
	[ $? -eq 0 ] && echo -e "\t\t ${USER_NAME}"
    done


}

intercept_change_password()
{
	local TARGET_SHADOW="/ffp/etc/shadow"

	USER_NAME="$1"
	
	[ -z "$USER_NAME" ] && USER_NAME=` whoami `
    
	/bin/busybox passwd "$@"
	local res=$?
	
	[ $res -ne 0 ] && exit $res
	
	SHADOW_ENTRY="$(grep "^${USER_NAME}:" /etc/shadow)"
	if [ -z "$SHADOW_ENTRY" ]; then
		echo "ERROR: Username not found in /etc/shadow?!" >&2
		exit 2
	fi

	 if grep -q "^${USER_NAME}:" /ffp/etc/shadow; then
		echo "Updating existing shadow entry in $TARGET_SHADOW..."
		PASSWD_HASH="$(echo "$SHADOW_ENTRY" |cut -d: -f2)"
		sed -i s,"^${USER_NAME}:[^:]*:","${USER_NAME}:${PASSWD_HASH}:", "$TARGET_SHADOW"
	else
		echo "Updating new shadow entry in $TARGET_SHADOW..."
		echo "$SHADOW_ENTRY" >>"$TARGET_SHADOW"
	fi
	
	exit 0
}

change_passwd()
{
	local user=$1
	local value=$2
	local valuename=$3
	local prefix="\tchange_$3 $1 $2:"
	local index=$4
	local passwd=/etc/passwd
	
	[ "${CHROOTDIR}" != "" ] && passwd=/ffp/etc/passwd # && echo -e "$prefix Not chrooted. Value will not be changed. Edit /ffp/etc/passwd instead" && return 0
	
	if [ "$ACTION" = "stop" ] ; then
		value=` read_db "$valuename $user" `
		[ "$value" = "" ] && echo -e "$prefix Value is not stored. Won't revert" && return 0
	fi
	
	local passwdline=` grep "^$user:" /etc/passwd `
	[ "$passwdline" = "" ] && echo -e "$prefix User not found in /etc/passwd" && return 0
	
	
	local current=` echo "$passwdline" | cut -d : -f $index `
	[ "$current" = "" ] && echo -e "$prefix Fatal error. Cannot read the current value of $user" && return 1
	[ "$current" = "$value" ] && echo -e "$prefix Value of $user is already $value" && return 0

	if [ "$ACTION" = "start" ] ; then
		if [ "$valuename" = "homedir" ] ; then
			[ -d $value ] || mkdir -m 700 -p $value && chown $user $value
		fi

		write_db "$valuename $user" $current
	fi
	
	local fmt
	if [ $index -eq 7 ] ; then
		fmt="\|^$user:|s|:$current\$|:$value|g"
	else
		fmt="\|^$user:|s|:$current:|:$value:|g"
	fi
	
	${SED} -i $fmt $passwd
	[ $? -eq 0 ] && echo -e "$prefix Changed $valuename of $user from $current to $value"
}

change_homedir()
{
	change_passwd $1 $2 homedir 6
}

change_shell()
{
	change_passwd $1 $2 shell 7
}

insert_line()
{
	local filename=$1
	local action=$2
	local tagline=$3
	local insert=$4
	local all=$5
	
	if [ "$action" = "before" ] ; then
		action=i
	else
		action=a
	fi
	
	local linenumbers=` grep -n "$tagline" $filename | sort -r | cut -d : -f 1 `
	
	[ "" = "$linenumbers" ] && return 1
	
	if [ "" = "$all" ] ; then
		linenumbers=` echo $linenumbers | tail -n 1 `
	fi
	
	local ret=0
	
	for l in $linenumbers
	do
		${SED} -i "${l}${action} $insert" $filename
		[ $? -ne 0 ] && ret=2
	done
	
	return $ret
}

PIDPREFIX=""

log_process_stuff()
{
	local pid=$1
	local ppid=` cat /proc/$pid/status | grep PPid | cut -d ':' -f 2 | sed 's|[\t| ]||g' `
	if [ "$ppid" != "" ] && [ -d /proc/$ppid ] ; then
		log_process_stuff $ppid
	fi
	
	echo ${PIDPREFIX}${pid}:` cat /proc/$pid/cmdline | tr '\000' ' ' ` >>/tmp/intercept.log.$$
	PIDPREFIX="$PIDPREFIX--"
}

log_interception()
{
	[ "" = "$INTERCEPT_LOG" ] && return 0
	
	local thepid=/tmp/zyxel_utils.pid
	local mypid=${thepid}.$$

	echo "**** ` whoami `@` date `:" >>/tmp/intercept.log.$$
	log_process_stuff $$

	local new_file=0
	# This function can be called multithreaded, so a lock is needed to keep the logfile consistent
	echo $$ >${mypid}
	trap "rm ${mypid}" EXIT
	
	while true
	do
		ln -s ${mypid} ${thepid} 2>/dev/null
		[ $? -eq 0 ] && break
		sleep 0.05
	done
	
	trap "rm ${thepid} ; rm ${mypid}" EXIT
	
	if [ ! -f $INTERCEPT_LOG ] ; then
		cat >>$INTERCEPT_LOG <<_EOF_
#==================================================================
# This file contains logs of all interceptions of firmware scripts
# as initiated by /ffp/start/zyxel_utils.sh
# You see timestamp and username for each call, and a backtrace of
# process calls (including PID of each process)
# If you don't want this logfile, edit  /ffp/start/zyxel_utils.sh
# to clear INTERCEPT_LOG
#=================================================================+
_EOF_
		new_file=1
	fi
	cat /tmp/intercept.log.$$ >>$INTERCEPT_LOG
	
	rm ${thepid} ; rm ${mypid}
	trap "" EXIT
	[ $new_file -eq 1 ] && chmod a+w $INTERCEPT_LOG
	rm /tmp/intercept.log.$$
}

intercept_shutdown()
{
	log_interception
	echo "SHUTTING_DOWN=true" >>/tmp/.ffpstick/helper.inc
	
	if [ "${FFP_TYPE}" != "zypkg" ] ; then
		exec exit_ffp
	fi
}

THIS_FILE=""

this_file()
{
	[ "$THIS_FILE" != "" ] && echo $THIS_FILE
	local curscript=$0
	if [ "`echo $curscript | head -c 1 `" != "/" ] ; then
	    curscript=` pwd `/$curscript
	fi
	local tmplink=/tmp/__tmp.${$}.tmp__
	ln -s $curscript $tmplink
	THIS_FILE=` readlink -f $tmplink `
	rm $tmplink
	echo $THIS_FILE
}

hook_shutdown()
{
	local rcshutdown=/etc/init.d/rc.shutdown
	local tf=` this_file `
	local cmd="$tf intercept shutdown"
	local prefix="\tclean_shutdown:"
	
	if [ "${CHROOTDIR}" != "" ] ; then
	    rcshutdown=/ffp/etc/original/init.d/rc.shutdown
	    cmd="chroot ${CHROOTDIR} $cmd"
	fi


	grep -i $tf $rcshutdown >/dev/null && echo -e "$prefix Clean shutdown is already active" && return 0
	
	insert_line $rcshutdown after zypkgs "${cmd}"
	case $? in
		1) 
			echo -e "$prefix Cannot find zypkgs stop in $rcshutdown" && return 1
			;;
		2)	
			echo -e "$prefix Error in inserting line" && return 2
			;;
	esac
		
	echo -e "$prefix Added clean shutdown to $rcshutdown"
}


intercept_samba()
{
	log_interception 

	local sambadir=/etc/samba/
	[ "$CHROOTDIR" != "" ] && sambadir=/ffp/etc/original/samba/
	[ "` cat ${sambadir}smb.conf | wc -l `" = "1" ] && return 1

	if [ ! -f /ffp/etc/samba/smb.conf ] ; then
		mkdir -p /ffp/etc/samba
		cp ${sambadir}smb.conf /ffp/etc/samba/
	fi
	
	mv ${sambadir}smb.conf ${STOREDIR}
	
	echo include = ${CHROOTDIR}/ffp/etc/samba/smb.conf >${sambadir}smb.conf
	return 0
}


KICKSAMBA=0

hook_samba()
{
	local sambash=/etc/init.d/samba.sh
	local sambadir=/etc/samba/
	local prefix="\thook_samba:"
	
	[ "$CHROOTDIR" != "" ] && sambash=/ffp/etc/original/init.d/samba.sh && sambadir=/ffp/etc/original/samba
	if [ "$ACTION" = "stop" ] ; then
		mv ${STOREDIR}smb.conf ${sambadir}
		[ "" != "` pidof smbd `" ] && KICKSAMBA=1
		return 0
	fi
	
	local tf=` this_file `
	local insertline="$tf intercept samba"
	if ! grep "$insertline" $sambash >/dev/null 2>&1 ; then
		[ "$CHROOTDIR" != "" ] && insertline="chroot $CHROOTDIR $insertline"
	
		for seek in start\) reload\)
		do
			insert_line $sambash after $seek "$insertline"
		done
		
		echo -e "$prefix Hooked $sambash"
	else
		echo -e "$prefix $sambash is already hooked"
	fi
	
	intercept_samba
	[ $? -eq 0 ] && [ "" != "` pidof smbd `" ] && KICKSAMBA=1
}

OUTCHROOT_PREFIX=""
RWDIR=""

hook_directory()
{
	local directory=$1
	local prefix="\thook_directory $directory:"

	if [ "$ACTION" = "stop" ] ; then
		${OUTCHROOT_PREFIX} umount $directory
		return 0
	fi
	
	if [ "" = "$RWDIR" ] ; then
		RWDIR=/ffp/var/rw/${FIRMWARE}/
		RWDIR=` echo "$RWDIR" | tr -d "()" `
	fi
	local backup=${RWDIR}${directory}
	
	if [ ! -d $backup ] ; then
		mkdir -p $backup
		cp -a $directory/* $backup/
	fi

	for file in ` cat /proc/mounts | cut -d ' ' -f 2 `
	do
		[ "$file" = "$directory" ] && echo -e "$prefix $directory is already mounted" && return 0
	done 

	${OUTCHROOT_PREFIX} mount --bind "${CHROOTDIR}/$backup" $directory
	[ $? -eq 0 ] && echo -e "$prefix done"
}

unhook_directory()
{
	local directory=$1
	local prefix="\tunhook_directory $directory:"
	${OUTCHROOT_PREFIX} umount $directory
	return 0
}

intercept_ftp()
{
	log_interception 
	
	local address=` read_db "ftp address" `
	
	# Address isn't stored?
	[ "" = "$address" ] && return 0

	local curargs=` cat /var/zyxel/pure-ftpd.arg `
	
	# Settings already applied
	echo "$curargs" | grep $address >/dev/null
	[ $? -eq 0 ] && return 0
	
	echo $curargs -P $address >/var/zyxel/pure-ftpd.arg
}

KICKFTP=0

hook_ftp()
{
	local address=$1
	local prefix="\thook_ftp $address:"
	
	hook_directory /usr/local/sbin
	[ "$ACTION" = "stop" ] && return 0

	write_db "ftp address" $address

	local tf=` this_file ` 
	local insertline="$tf intercept ftp"
	[ "$CHROOTDIR" != "" ] && insertline="chroot $CHROOTDIR $insertline"
	
	local changed=0
	
	for file in /usr/local/sbin/vsftpd_start.sh /usr/local/sbin/vsftpd_start_silent.sh
	do
		file=${RWDIR}${file}
		if [ -f $file ] ; then
			if grep zyxel_utils $file >/dev/null
			then
				echo -e "$prefix $file is already hooked"
			else
				insert_line $file before /bin/nice "$insertline" all
				[ $? -eq 0 ] && changed=1
			fi
		else
			echo -e "$prefix Cannot find $file" 
		fi
	done
	
	if ps -A | grep pure-ftpd >/dev/null
	then
		KICKFTP=1
	fi
	
	echo -e "$prefix Done"
}

KICKCROND=0
CRONTAB=/var/spool/cron/crontabs/root

hook_crond()
{
	local prefix="\thook_crond:"

	if [ "$ACTION" = "stop" ] ; then
		unhook_any /sbin/crond
		[ $? -ne 0 ] && echo -e "$prefix Not hooked" && return 0
		
		cat $CRONTAB | grep -v FFPHOOK >$CRONTAB.$$
		mv $CRONTAB.$$ $CRONTAB
		echo -e "$prefix Unhooked"
	else
		sed -i 's|^\#/bin/sh|\#\!/bin/sh|g' /etc/init.d/crond.sh
		hook_any /sbin/crond
		[ $? -ne 0 ] && echo -e "$prefix Is already hooked" && return 0

		echo -e "$prefix Done"
	fi

	KICKCROND=1
}

intercept_crond()
{
	cat $CRONTAB | grep -v FFPHOOK >$CRONTAB.$$
	echo "FFPHOOK=" >$CRONTAB
	cat $CRONTAB.$$ >>$CRONTAB
	rm $CRONTAB.$$
	
	if [ -f /ffp/etc/crontab ] ; then
		while read line
		do
			echo "$line \$FFPHOOK" >>$CRONTAB
		done </ffp/etc/crontab
	fi
}

intercept_nfs()
{
	log_interception 
	
	local desired_exports="${CHROOTDIR}/ffp/etc/exports"

	[ "` read_db "nfs hook" `" != "on" ] && return 0
	
	if [ "$1" = "off" ] ; then
		desired_exports=/usr/local/zy-pkgs/etc/exports 
	else
		[ ! -f $desired_exports ] && cp /etc/exports $desired_exports
	fi
	
	rm /etc/exports
	ln -s $desired_exports /etc/exports
}

KICKNFS=0
NFSSCRIPT="/usr/local/zy-pkgs/etc/init.d/NFS"

hook_nfs()
{
	local prefix="\thook_nfs:"
	local tf=` this_file `

	[ ! -x $NFSSCRIPT ] && echo -e "$prefix $NFSSCRIPT is not executable" && return 0

	if [ "$ACTION" = "stop" ] ; then
		write_db "nfs hook" off
	else
		local insertline="$tf intercept nfs"
		[ "$CHROOTDIR" != "" ] && insertline="chroot $CHROOTDIR $insertline"
	
		cat $NFSSCRIPT | grep "$insertline" >/dev/null
		if [ $? -eq 0 ] ; then
			echo -e "$prefix $NFSSCRIPT is already hooked"
		else
			# Special care: This script can outlive ffp
			insertline="[ -x ${CHROOTDIR}${tf} ] && $insertline"
			insert_line $NFSSCRIPT after "start_nfs()" "$insertline on"
			insert_line $NFSSCRIPT after "\${NICE_VALUE} sm-notify" "$insertline off"
		fi
	
		write_db "nfs hook" on
	fi
	
	if ps -A | grep nfsd >/dev/null
	then
		KICKNFS=1
	fi

	echo -e "$prefix Done"
}

hook_any()
{
	local to_hook=$1
	
	if [ -d $to_hook ] ; then
		hook_directory $to_hook
		return $?
	fi
	
	local prefix="\thook_any $to_hook:"
	local file=` basename ${to_hook} `
	local dir=` dirname ${to_hook} `
	
	mkdir -p ${dir}/hooked
	[ $? -ne 0 ] && echo -e "$prefix Cannot create ${dir}/hooked/. (Read only dir? Try to hook it)" && return 1
	
	local hooked=${dir}/hooked/${file}
	[ -x ${hooked} ] && echo -e "$prefix $to_hook is already hooked." && return 2

	local curscript=` this_file `

	if [ "" = "` readlink ${to_hook} `" ] ; then
		# No symlink, it's a 'real binary (tm)
		mv ${dir}/${file} ${hooked}
		[ $? -ne 0 ] && echo -e "$prefix Cannot move $to_hook." && return 3
	else
		ln -s ` readlink -f ${to_hook} ` ${hooked}
		[ $? -ne 0 ] && echo -e "$prefix Cannot create symlink $hooked." && return 4
		rm ${dir}/${file}
		[ $? -ne 0 ] && echo -e "$prefix Cannot remove ${dir}/${file}." && return 5
	fi

	ln -s $curscript  ${dir}/${file}
	[ $? -ne 0 ] && echo -e "$prefix Cannot create symlink ${dir}/${file}" && return 6
	return 0
}

unhook_any()
{
	local to_hook=$1

	if [ -d $to_hook ] ; then
		unhook_directory $to_hook
		return $?
	fi
	
	local prefix="\tunhook_any $to_hook:"
	local file=` basename ${to_hook} `
	local dir=` dirname ${to_hook} `

	local hooked=${dir}/hooked/${file}
	
	[ ! -x ${hooked} ] && echo -e "$prefix $to_hook isn't hooked" && return 1
	
	rm ${to_hook}
	mv ${hooked} ${to_hook}
}

intercept_any()
{
	log_interception
	
	local file=` basename $1 `
	local dir=` dirname $1 `
	shift
	
	if [ "${file}" = "crond" ] ; then
		intercept_crond
	fi
	
	if [ "${file}" = "change_password" ] ; then
		intercept_change_password $@
		exit $?
	fi
	
	exec ${dir}/hooked/${file} $@
}

init()
{
	if [ -f /tmp/.ffpstick/helper.inc ] ; then
		. /tmp/.ffpstick/helper.inc
	else
		echo -e "\tFatal: Not a ZyXEL"
		exit 1
	fi

	if [ "$SHUTTING_DOWN" = "true" ] ; then
		echo -e "\tShutting down, Skipping" && exit 0
	fi 
	
	if [ "${CHROOTDIR}" != "" ] ; then
	    OUTCHROOT_PREFIX=execute_outside_chroot
	fi
}

finalize()
{
	if [ $KICKWEBS -eq 1 ] ; then
		echo -e "\tRestarting apache..."
		local httpdpid=` pidof httpd `
		[ "$httpdpid" != "" ] && kill -HUP $httpdpid
	fi
	
	if [ $KICKSAMBA -eq 1 ] ; then
		echo -e "\tRestarting samba..." 
		${OUTCHROOT_PREFIX} /etc/init.d/samba.sh restart
	fi
	
	if [ $KICKFTP -eq 1 ] ; then
		echo -e "\tRestarting pure-ftpd..."
		${OUTCHROOT_PREFIX} /usr/local/sbin/vsftpd_start_silent.sh
	fi

	if [ $KICKNFS -eq 1 ] ; then
		echo -e "\tRestarting nfsd..."
		${OUTCHROOT_PREFIX} $NFSSCRIPT restart
	fi

	if [ $KICKCROND -eq 1 ] ; then
		echo -e "\tRestarting crond..."
		${OUTCHROOT_PREFIX} /etc/init.d/crond.sh restart
	fi
	
	if [ $ACTION = "stop" ] ; then
		rm $DBFILE
	fi
}

if [ "` basename $0 `" != "zyxel_utils.sh" ] ; then
	intercept_any $0 $@
fi


case $1 in
	start)
		echo Starting $name
		init
		ACTION=start
		zyxel_utils start
		finalize
		;;
	stop)
		echo Stopping $name
		init
		ACTION=stop
		zyxel_utils stop
		finalize
		;;
	intercept)
		init
		case $2 in
			samba)
				intercept_samba
				;;
			ftp)
				intercept_ftp
				;;
			nfs)
				intercept_nfs $3
				;;
			shutdown)
				intercept_shutdown 
				;;
		esac
		;;
	status)
		;;
	hook)
		hook_any $2
		[ $? -ne 0 ] && echo Failed to hook $2
		;;
	unhook)
		unhook_any $2
		[ $? -ne 0 ] && echo Failed to unhook $2
		;;
	*)
		cat  <<_EOF_
Usage $0 [start|stop|hook (executable|directory)|unhook (executable|directory)]
_EOF_
		;;
esac

